/****************************************************************************
** IR1 Library Header File.
*****************************************************************************
*/

#ifndef __ir1_h
#define __ir1_h

#ifdef   __cplusplus
extern "C" {
#endif

/*-------------------------------------
**  dislist structs & defines
*/
#define DIRLIST_MAX_LIST  20
#define DIRLIST_MAX_PATH  256

struct dirlist_t
{
	char path[DIRLIST_MAX_PATH];
	int  nele;
	char *list[DIRLIST_MAX_LIST];
};

/*-------------------------------------
**  strings...
*/
char * btosl ( char * s,  unsigned long l );
char * btoss ( char * s,  unsigned short l );
int  matchname( char *pattern, char *name);
int  my_atol( char *s, long *l);
int  str_replace_sub( char *src, char *dest, int dest_size, char *pattern, char *val);
char *str_rws( char *s, unsigned int maxlen);
int stricmp( char *s, char *t);
char *strlwr( char * string);
char *strupr( char * string);
char *strxcat( char *dest, char *src, int dest_size);
char *strxcpy( char *dest, const char *src, int maxlen);
char *unpad( char *s, int c);

int dateStr2unixtime( double *tv_sec, char *str);

int timeStr2sec( double *Rsec, char *str);
char * sec2timeStr ( double sec, int decimals);

/*-------------------------------------
**  files & directories ...
*/
char *cat_pathname( char *pathname, char *path, char *filename, int maxlen);
int create_path( char * p);
void dir_from_path( char *dir, char *pathname, int dir_size );
void dirlist_free( struct dirlist_t *dl );
void dirlist_init( struct dirlist_t *dl );
void dirlist_makelist( struct dirlist_t * dl );
void dirlist_newpath( struct dirlist_t *dl, char *path );
void dirlist_showlist( struct dirlist_t * dl );
int exist_path( char * pathname );
int expand_pathname(  char *pathout, int  pathout_size, char *pathin);
void filename_from_path( char *filename, char *pathname, int filename_size );
int get_full_pathname( char * pathout, int pathout_size, char *pathin);

/*-------------------------------------
**  socket...
*/
int sock_open( char * hostname, int port );
long sock_read_data( int fd, char *buf, long bufsize, int blocking, int socket_timeout_cnt);
int sock_read_msg( int fd, char *msg, int blocking, int socket_timeout_cnt);
int sock_setup_server( int port_no );
long sock_write_data( int fd, char *buf, long bufsize, int blocking, int socket_timeout_cnt);
int sock_write_msg( int fd, char *msg, int blocking, int socket_timeout_cnt);

/*-------------------------------------
**  ipc...
*/
int msgtran( int key, int create_it);
int shmtran( int key, int nbytes, int create_it);
int semtran( int key, int create_it);
int sem_op( int sid, int value );

/*-------------------------------------
**  io...
*/
void set_keypress(void);
void reset_keypress(void);

/*--------------------------------------
** string parsing & conversion
*/
int parseInt( int * ip, char *buf, const char *tok );
int parseIntR( int * ip, char *buf, const char *tok, int min, int max );
int parseLong( long * lp, char *buf, const char *tok );
int parseLongR( long * lp, char *buf, const char *tok, long min, long max );
int parseDouble( double *dp, char *buf, const char *tok );
int parseDoubleR( double *dp, char *buf, const char *tok, double min, double max);
int parseFloat( float *fp, char *buf, const char *tok );
int parseFloatR( float *fp, char *buf, const char *tok, float min, float max );
int parseSelection( char * buf, char * tok, char ** list);
int parseString( char *outb, int outb_size, char *buf, const char *tok );

int parseInt_r( int * ip, char *buf, const char *tok, char **st_ptr );
int parseIntR_r( int * ip, char *buf, const char *tok, char **st_ptr, int min, int max );
int parseLong_r( long * lp, char *buf, const char *tok, char **st_ptr );
int parseLongR_r( long * lp, char *buf, const char *tok, char **st_ptr, long min, long max );
int parseDouble_r( double *dp, char *buf, const char *tok, char **st_ptr );
int parseDoubleR_r( double *dp, char *buf, const char *tok, char **st_ptr, double min, double max);
int parseFloat_r( float *fp, char *buf, const char *tok, char **st_ptr );
int parseFloatR_r( float *fp, char *buf, const char *tok, char **st_ptr, float min, float max );
int parseSelection_r( char * buf, char * tok, char **st_ptr, char ** list);
int parseString_r( char *outb, int outb_size, char *buf, const char *tok, char **st_ptr );

/*-----------------------------------------------------------------
** Types and prototype for 2D coordinate transformation functions
** See t2d.c for details.
*/
typedef double t2d_matrix[3][3];   /* matrix type */
typedef double t2d_vector[3];      /* vector type */

int t2d_tranlation( t2d_matrix m, double tx, double ty );
int t2d_rotation( t2d_matrix m, double deg );
int t2d_scaling( t2d_matrix m, double sx, double sy );

int t2d_forward_matrix( t2d_matrix m,  double Tx, double Ty, double Sx, double Sy, double R);
int t2d_reverse_matrix( t2d_matrix m, double Tx, double Ty, double Sx, double Sy, double R);

int  t2d_apply_to_point( double *dx, double *dy, t2d_matrix m, double sx, double sy);

void t2d_m_mul( t2d_matrix p, t2d_matrix a, t2d_matrix b);
void t2d_m_copy(t2d_matrix dst, t2d_matrix src);
void t2d_m_identity(t2d_matrix m);
void t2d_m_print(t2d_matrix m);

/*--------------------------------------
** others
*/
double dlimit( double v, double min, double max);
double drange( double v, double min, double max);
double dpoly( double x, double *coeff, int degree );
void get_vector( int * dir, double *mag, double start, double dest, double size );

double map( double point, double pref1, double pref2, double nref1, double nref2 );
int point_in_polygon( double   pgon[][2], int   numverts, double   point[2]);

long elapse_ms( struct timeval *then);
float elapse_sec( struct timeval *start, struct timeval *end);
long my_timegm( int year, int yday, int hr, int min, int sec);

/*--------------------------------------------------------------------------
** defines related to socket port number and IPC ID, etc.
**--------------------------------------------------------------------------
*/
#define TCS_MSG_PORT      30010    /* Port ID for TCS 'normal' communications */
#define TCS_AUX_PORT      30011    /* Port ID for TCS telnet communications */

#define HEXED_PORT        30015    /* Port ID for Hexed communications */

 
#define IRTF_VIEWER_MSGQ    30120    /* Message Queue name for IRTF vf program */
#define MIRLIN_VIEWER_MSGQ  30121    /* Message Queue name for MIRLIN mvf program */

#define IRTF_VIEWER_PORT      30122    /* Port ID for IRTF Viewer program (vf)   */
#define IRTF_DV_PORT          30123    /* Port ID for IRTF Viewer program (dv)   */
#define IRTF_DV_BIGDOG_PORT   30124    /* Port ID for IRTF Viewer program (dv)   */
#define IRTF_DV_GUIDEDOG_PORT 30125    /* Port ID for IRTF Viewer program (dv)   */
#define IRTF_TEST_PORT        30126    /* Port ID for Application development    */
#define MIRLIN_VIEWER_PORT    30127    /* Port ID for MIRLIN Viewer (mvf)        */
 
#define ECHELLE_MSG_PORT  30120    /* Port ID for ECHELLE spectrograph      */
#define ECHELLE_DATA_PORT 30121

#define CSHELL_MSG_PORT   30120     /* Cshell II reuses ECHELLE's ports      */
#define CSHELL_DATA_PORT  30121
 
#define NSFCAM_MSG_PORT   30140    /* Port ID for NSFCAM camera             */
#define NSFCAM_DATA_PORT  30141

#define GUIDER_MSG_PORT   30145    /* Port ID for GUIDER camera             */
#define GUIDER_DATA_PORT  30146
 
#define MIRLIN_MSG_PORT     30147    /* Port ID for MIRLIN camera             */
#define MIRLIN_DATA_PORT    30148

#define SOCKET_MSG_LEN      160    /* Size of message string used in sockets */
#define PACKET_SIZE       32768    /* Max num of bytes to socket write/read  */

/*--------------------------------------
**  Very common definitions.
**--------------------------------------
*/

#ifndef TRUE
#define TRUE    1
#define FALSE   0
#define ON      1
#define OFF     0
#endif
  
#ifndef INRANGE
#define INRANGE(a,x,b)   ((a) <= (x) && (x) <= (b))
#endif
	
#ifndef MAX
#define MAX( a, b )      ((a) < (b) ? (b) : (a))
#define MIN( a, b )      ((a) < (b) ? (a) : (b))
#endif
	 
#ifndef isodd
#define isodd(a)         ((a) % 2)
#define iseven(a)        (!((a) % 2))
#endif

#define P( a ) 			sem_op( a, -1)
#define V( a ) 			sem_op( a,  1)

/*------------------------------------------------------------------
**  Error codes 
**------------------------------------------------------------------
*/

/* this funtions return error string for any ERR_* code */
char * error_string( int rc );

#define ERR_NONE                    (0)    /* No error        */
#define ERR_INV_KW                 (-1)    /* Invalid keywork */
#define ERR_INV_RNG                (-2)    /* Invalid Range   */
#define ERR_INV_FORMAT             (-3)    /* Invalid Format or syntax error */
#define ERR_SOCKET_ERR            ( -4)    /* Socket communication error     */
#define ERR_SOCKET_TIMEOUT        ( -5)    /* Socket communication error     */
#define ERR_INV_PATH              ( -6)    /* This is not a valid path       */
#define MEM_ALT_ERR               ( -7)    /* Memory allocation error        */
#define ERR_FILE_CREATE           ( -8)    /* Error Creating File            */
#define ERR_FILE_WRITE            ( -9)    /* Error writing file             */
#define ERR_FILE_READ             (-10)    /* Error reading a file           */
#define ERR_FILE_FORMAT           (-11)    /* Invalid file format            */
#define ERR_NOT_READY             (-12)    /* Unable to execute, object is busy    */
#define ERR_BUF_EMPTY             (-13)    /* Unable to perform on an empty buffer */
#define ERR_BUF_DIFF_SIZE         (-14)    /* Unable to perform on different size buffers */
#define ERR_INV_BUF               (-15)    /* Invalid buffer specified       */
#define ERR_INV_OPERATION         (-16)    /* This is an invalid operation   */
#define ERR_UNKNOWN               (-17)    /* This shouldn't happen          */
#define ERR_RESTRICTED            (-18)    /* This option is currently restricted */
#define ERR_MSGQ                  (-19)    /* System error on message queues */
#define ERR_STOP                  (-20)    /* The GO was interrupted by a stop command */
#define ERR_GRATING_FAILURE       (-21)    /* Grating Task Failure         */
#define ERR_FILE_OPEN             (-22)    /* Error Opening File            */
#define ERR_LIMIT_FAIL            (-23)    /* Some limit switch is broken */
#define ERR_SEM_GET               (-24)    /* Semaphore acquisition error */
#define ERR_NOT_INITIALIZED       (-25)    /* Device was not properly initialized */
#define ERR_DEVICE_IO             (-26)    /* Error communication with device     */
#define ERR_INV_COMBINATION       (-27)    /* Error combination specified     */
#define ERR_TIMEOUT               (-28)    /* A Timeout error                 */
#define ERR_INVALID_DIR           (-29)    /* This director is invalid/inaccessible   */
#define ERR_CLOCK_DSP             (-30)    /* Clock DSP error */
#define ERR_BCARD_DSP             (-31)    /* Bcard DSP error */
#define ERR_NOT_IMPLEMENTED       (-32)    /* This option is not implement */
#define ERR_SUBARRAY_CNT          (-33)    /* Subarray count not valid  */
#define ERR_SUBARRAY_ROW          (-34)    /* Subarray row error       */
#define ERR_SUBARRAY_FORMAT       (-35)    /* Error in Format or placement of subarray */
#define ERR_SUBARRAY_TOOBIG       (-36)    /* Error in Format or placement of subarray */
#define ERR_ENCODER               (-37)    /* Encoder error       */
#define ERR_TCLAMP_ON             (-38)    /* Thermal clamp is locked */
#define ERR_NOT_AVAILABLE         (-39)    /* Reqested action or service is not available */
#define ERR_SHIFT_ARRAY           (-40)    /* Shift array is wrong */
#define ERR_LIMITATION            (-41)    /* Some sort of limit was exceeded */
#define ERR_HARDWARE              (-42)    /* Hardware didn't work as it should */
#define ERR_NO_DATA               (-43)    /* No data */
#define ERR_ABORT                 (-44)    /* Last operation was aborted */
#define ERR_NA45                  (-45)    /* undefined */
#define ERR_NA46                  (-46)    /* undefined */
#define ERR_NA47                  (-47)    /* undefined */
#define ERR_NA48                  (-48)    /* undefined */
#define ERR_NA49                  (-49)    /* undefined */
#define ERR_DSP_EXECUTE           (-50)
#define ERR_DSP_UNKNOWN           (-51)   /* Opps, don't know what happen. */
#define ERR_DSP_DEVICE_IO         (-52)   /* error communicating with device */
#define ERR_DSP_INV_INPUT         (-53)   /* Invalid input detected, ie out of range */
#define ERR_DSP_INV_FUN           (-54)   /* invalid function */
#define ERR_DSP_NOT_READY         (-55)   /* not ready to perform this function */
#define ERR_DSP_SHARC             (-56)   /* The board isn't working */
#define ERR_DSP_IXR_LOCK          (-57)   /* error related to locking the IXR */
#define ERR_DSP_FILE_READ         (-58)   /* Error reading input file for DSP function */
#define ERR_DSP_SUBARRAY_FORMAT   (-59)   /* invalid format for subarrays */
#define ERR_NA60                  (-60)   /* undefined */

#define NUM_ERR_CODES               61    /* number of defined codes. */

/****************************************************************************
** df_ functions are Double Fits Library.
*****************************************************************************
*/

/* FITS BITPIX values for data types */
#define DF_BITPIX_LONG         32
#define DF_BITPIX_SHORT        16
#define DF_BITPIX_FLOAT       -32
#define DF_BITPIX_DOUBLE       64
#define DF_BITPIX_CHAR          8

#define DF_FITS_RECORD_LEN   2880  /* Fits format has 2880 bytes per record  */
#define DF_FITS_LEN_FILENAME   20
#define DF_FITS_LEN_PATH       80
 
#define DF_MAX_SIGNED_LONG   ((long)0x7FFFFFFFL)
#define DF_MIN_SIGNED_LONG   ((long)0x80000001L)
 
#define DF_EMPTY      0          /* File or Buffer status                  */
#define DF_UNSAVED    1
#define DF_SAVED      2

#define DF_MATH_ADD   0          /* Operation code for df_math() function  */
#define DF_MATH_SUB   1
#define DF_MATH_MUL   2
#define DF_MATH_DIV   3
#define DF_MATH_COPY  4
#define DF_MATH_SQRT  5

#define DF_ROT_M90    0
#define DF_ROT_P90    1
#define DF_ROT_180    2

#define DF_ORIGIN_TL  0 /* (0,0) is at the top left */
#define DF_ORIGIN_BL  1 /* (0,0) is at the bottom left */

/*--------------------------------------------------------
**  DF structures for holding data and sending fits data
*/
 
union u_df_lf               /* for long/float converstion with ntohl() */
{
   unsigned long l;
   float f;
};

struct df_fheader_t
{
   struct df_fheader_t * next;
   char               buf[DF_FITS_RECORD_LEN];
};

 
struct df_internal_vars
{
   int divbycoadd;   /* should the divisor be applied to the data? */
   int origin;       /* where is the origin */
};


struct df_buf_t
{
   short              status;        /* Buffer status: empty, unsaved,...  */
   short              naxis1;        /* Number of points in NAXIS1         */
   short              naxis2;        /* Number of points in NASIS2         */
   short              size;          /* sizeof data value in bytes         */
   short              bitpix;        /* BITPIX code                        */
   long               N;             /* Number of data points              */
   float              max;           /* max data value                     */
   float              min;           /* min data value                     */
   float              mean;          /* mean data value in frame           */
   float              stddev;        /* STD of data in frame               */
   float              arcsec_pixel;  /* Number of arcseconds per pixel     */
	float              pos_angle;     /* position angle of image            */
   float              divisor;       /* Used by divbycoadds.               */
   float              itime;         /* itime */
	float              filter_zp;     /* filter zero point - default = 0    */
	float              ext_coff;      /* extinction co-efficient. default=0 */
	float              airmass;       /* airmass. default=0                 */
   short              Nheader;       /* Number of lines in the fits hdr.   */

	short              org_size;      /* original sizeof(pixel_data) in bytes */
	short              org_bitpix;    /* original bitpix value */
	float              org_bscale;    /* original bscale */
	float              org_bzero;     /* original bzero */
	long               sizeof_header; /* sizeof header in the file. */
	long               nframes;       /* number of movie frames ie: naxis3 */

   short              gbox_enable;   /* Is a guidebox associated with this data */
   long               gbox_dim[4];   /* guidebox x,y,wid,hgt */
   float              gbox_from[4];  /* guidebox from x,y */
   float              gbox_to[4];    /* guidebox to x,y */

   char               directory[DF_FITS_LEN_PATH];
   char               filename[DF_FITS_LEN_FILENAME];
   struct df_fheader_t * fheader;      /* Pointer to header block.           */
   float *            fdata;        /* Pointer to block of data           */
};

/*------------------------------------------------------------------
**  df_ Prototypes
**------------------------------------------------------------------
*/
int df_init( void );
int df_print_options( void );
int dfset_divbycoadd( int divbycoadd );
int dfget_divbycoadd( void );
int dfset_origin( int origin );
double dfdatamn( struct df_buf_t * bufp, int m, int n );
double dfdataxy( struct df_buf_t * bufp, int x, int y );
double dfdatainx( struct df_buf_t * bufp, int i );
double df_data_f( struct df_buf_t * bufp, int i );

int df_write_fits( int fd, char *path, char *filename, struct df_buf_t *bufp );

int df_read_fits( int fd, char *path, char *filename, struct df_buf_t *bufp, 
	int is_socket, int is_3d );

int df_search_fheader( struct df_fheader_t *hdr, char *keyword, char *val_str,
						  int val_str_size, struct df_fheader_t **Rhdr, int *offset, int debug);
int df_free_fbuffer( struct df_buf_t * bufp);

char * df_build_card( char *cptr, char *keyword, char *value, char *comment);

int df_buffer_math( struct df_buf_t * dest, struct df_buf_t * op1, 
						  struct df_buf_t * op2, int operation ); 
int df_constant_math( struct df_buf_t * dest, struct df_buf_t * op1, 
						  float op2, int operation ); 
int df_buffer_rotate( struct df_buf_t * dest, struct df_buf_t * op1, int operation);
int df_copy_subarray( struct df_buf_t * dest, struct df_buf_t * op1, 
							 int op_x, int op_y, int op_wid, int op_hgt);
int df_buffer_userfun( struct df_buf_t * dest, struct df_buf_t * op1,
	int (*userfun) ( struct df_buf_t *dest, struct df_buf_t *op1 ));
int df_math_copyheader( struct df_buf_t * dest, struct df_buf_t * src);
int df_math_fixheader( struct df_buf_t * bufp, char * history);

int df_stats( struct df_buf_t *bufp );
int df_stats_p1( struct df_buf_t *bufp );
int df_stats_p2( struct df_buf_t *bufp, double mean );

/****************************************************************************
** cbl related defines. CBL is Command Buffer List 
*****************************************************************************
*/

#define CBL_MAX_CMD_LEN    160

/* Special list commands */
#define CBL_CMD_COMMIT     0x80000000
#define CBL_CMD_UP         0x80000001
#define CBL_CMD_DOWN       0x80000002
#define CBL_CMD_LEFT       0x80000003
#define CBL_CMD_RIGHT      0x80000004
#define CBL_CMD_DELETE     0x80000005
#define CBL_CMD_BACKSPACE  0x80000006

/* Does the value fit the list command format? */
#define CBL_IS_VALID_LIST_CMD(c)   ((c) & 0x80000000))

/* Examine an error code to determine if it represents a failure value */
#define CBL_FAILED(e)      (((unsigned)(e)) & 0x80000000)

/* List entry */
typedef struct _cbl_entry
{
   char   cmd[CBL_MAX_CMD_LEN + 1];
}cbl_entry;

/* Command buffer list */
typedef struct _cbl_list
{
   cbl_entry*  entries;     // Committed entries
   cbl_entry   edited;      // The entry being edited now
   int  browsing;           // T/F, Is the list is being browsed?
   int  new_command;        // T/F, Is there a new command?
   long current;            // Index of the current entry
   long count;              // Number of elements in the list
   long max_entries;        // Maximum number of entries in the list
   long entry_cursor;       // The position of the insertion character for the current entry
} cbl_list;

/*--------------------------------
** cbl prototypes 
*/

cbl_list *cbl_new(long max_entries);
void cbl_destroy(cbl_list *pThis);
long cbl_get_max_entries(const cbl_list *pThis);
long cbl_get_count(const cbl_list *pThis);
int  cbl_has_new_command(const cbl_list *pThis);
void cbl_get_last_command(cbl_list *pThis, char *buf, int buf_size );
int  cbl_get_command(const cbl_list *pThis, long index, char *buf, int buf_size );
void cbl_get_current_command(const cbl_list *pThis, char *buf, int buf_size );
void cbl_set_current_command( cbl_list *pThis, const char *text);
long cbl_get_entry_cursor(const cbl_list *pThis);
void cbl_set_entry_cursor( cbl_list *pThis, long cursor);
int  cbl_handle_command( cbl_list *pThis, int command);

/****************************************************************************
** clo related defines. CBL is Command Line Option function.
*****************************************************************************
*/

// clo options structure
struct clo_option_t
{
   int  type;          // CLO_TYPE_ - type of argument
   char * flag;        // command line flag string, ie: "-v"
   char * default_str; // Default value, as a sting
   void * user_var;    // pointer to user variable to hold option's value (must match type)
   int  sizeof_var;    // size of user_var (only important for char [])
};

// clo defines
#define CLO_TYPE_NO_ARG  0       // no argument - boolean type (assume int).
#define CLO_TYPE_LONG    1       // long    - 32bit int
#define CLO_TYPE_DOUBLE  2       // double  - double float.
#define CLO_TYPE_STRING  3       // char    - character string

// clo function prototypes
int clo_parse( int *argc, char *argv[], struct clo_option_t clo_options[], int num_clo_options);


#ifdef   __cplusplus
}
#endif

#endif /* __ir1_h */

