
#define EXTERN

/*-------------------------
**  Standard include files
**-------------------------
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <math.h>
#include <sys/time.h>

/*------------------------------
**  Non-standard include files
**------------------------------
*/

#include "ir1.h"

/*--------------------------------------------------------------------
**  timeStr2Sec() - convert a string in time format "hh:mm:ss.ss"
**      to seconds (double).
**--------------------------------------------------------------------
*/
int timeStr2sec(
   double *Rsec,       /* Returns seconds to the caller */
   char   *str         /* string in time format (hh:mm:ss.ss) to convert */
   )
{
   char time[30];
   int hh, mm;
   float ss, sign;
   char * cptr;
   char * st_ptr; // for strtok_r()

   if( str == NULL )
      return ERR_INV_FORMAT;

   strxcpy( time, str, sizeof(time));

   sign = 1;    /* assume default of positive time value */

   if( NULL == (cptr = strtok_r( time, " :", &st_ptr)) )
      return( ERR_INV_FORMAT );
	if( strchr( cptr, '-') )
		sign = -1;
	hh = atoi(cptr);
	if( hh < 0 )
	{
		hh = -hh;
		sign = -1;
	}

   if( NULL == (cptr = strtok_r( (char*)NULL, " :", &st_ptr)) )
      return( ERR_INV_FORMAT );
   mm = ( cptr == NULL ? 0 : atoi(cptr) );

   if( NULL == (cptr = strtok_r( (char*)NULL, " :", &st_ptr)) )
      return( ERR_INV_FORMAT );
   ss = ( cptr == NULL ? 0 : atof(cptr) );

   *Rsec = (3600.0*hh + 60.0*mm + ss) * sign;
   return ERR_NONE;
}

/*----------------------------------------------------------------------
** sec2TimeStr() convert a double (number of seconds) into a string
**    with the format "hh:mm:ss.ss".
**    Return the the address containing the formatted string.
**----------------------------------------------------------------------
*/
char * sec2timeStr (
   double sec,       /* convert test seconds to a string */
   int decimals      /* number of decmial points for ss.ss */
)
{
   static char buf[30];

   char fmt[30];
   int  hh, mm, sign;

   if( sec < 0 )
   {
      sec = -sec;
      sign = -1;
   }
   else
      sign = 1;

   hh = (int) floor( sec / 3600.0 );
   sec -= 3600.0 * hh;

   mm = (int) floor( sec / 60.0 );
   sec -= 60.0 * mm;

   if( decimals <= 0 )
      sprintf( buf, "%02d:%02d:%2.0f", sign*hh, mm, sec);
   else
   {
      sprintf( fmt, "%%02d:%%02d:%%2.%df", decimals);
      sprintf( buf, fmt, sign*hh, mm, sec);
   }
   return buf;
}

